@@ -20,10 +20,16 @@ from typing import Dict, Tuple, Union
 import numpy as np
 import torch
 from ignite.contrib.handlers.tensorboard_logger import (
-    OutputHandler, TensorboardLogger, global_step_from_engine)
+    OutputHandler,
+    TensorboardLogger,
+    global_step_from_engine,
+)
 from ignite.contrib.handlers.tqdm_logger import ProgressBar
-from ignite.engine import (Events, create_supervised_evaluator,
-                           create_supervised_trainer)
+from ignite.engine import (
+    Events,
+    create_supervised_evaluator,
+    create_supervised_trainer,
+)
 from ignite.handlers import EarlyStopping, ModelCheckpoint
 from ignite.metrics.loss import Loss
 from torch.nn import Module
@@ -37,9 +43,7 @@ from neural_semigroups.magma import Magma
 
 
 def load_database_as_cubes(
-        cardinality: int,
-        train_size: int,
-        validation_size: int
+    cardinality: int, train_size: int, validation_size: int
 ) -> Tuple[
     np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray
 ]:
@@ -61,15 +65,20 @@ def load_database_as_cubes(
     for cayley_table in tqdm(train.database, desc="generating train cubes"):
         train_cubes.append(Magma(cayley_table).probabilistic_cube)
     validation_cubes = list()
-    for cayley_table in tqdm(validation.database,
-                             desc="generating validation cubes"):
+    for cayley_table in tqdm(
+        validation.database, desc="generating validation cubes"
+    ):
         validation_cubes.append(Magma(cayley_table).probabilistic_cube)
     test_cubes = list()
     for cayley_table in tqdm(test.database, desc="generating test cubes"):
         test_cubes.append(Magma(cayley_table).probabilistic_cube)
     return (
-        np.stack(train_cubes), np.stack(validation_cubes),
-        np.stack(test_cubes), train.labels, validation.labels, test.labels
+        np.stack(train_cubes),
+        np.stack(validation_cubes),
+        np.stack(test_cubes),
+        train.labels,
+        validation.labels,
+        test.labels,
     )
 
 
@@ -86,50 +95,50 @@ def get_arguments() -> Namespace:
         type=int,
         help="magma cardinality",
         required=True,
-        choices=range(2, 8)
+        choices=range(2, 8),
     )
     parser.add_argument(
         "--epochs",
         type=int,
         help="number of epochs to train",
         default=100,
-        required=False
+        required=False,
     )
     parser.add_argument(
         "--learning_rate",
         type=float,
         help="learning rate",
         default=0.001,
-        required=False
+        required=False,
     )
     parser.add_argument(
         "--batch_size",
         type=int,
         help="batch size for training",
         default=32,
-        required=False
+        required=False,
     )
     parser.add_argument(
         "--train_size",
         type=int,
         help="number of tables for training",
-        required=True
+        required=True,
     )
     parser.add_argument(
         "--validation_size",
         type=int,
         help="number of tables for validation",
-        required=True
+        required=True,
     )
     return parser.parse_args()
 
 
 def learning_pipeline(
-        params: Dict[str, Union[int, float]],
-        cardinality: int,
-        model: Module,
-        loss: Loss,
-        data_loaders: Tuple[DataLoader, DataLoader],
+    params: Dict[str, Union[int, float]],
+    cardinality: int,
+    model: Module,
+    loss: Loss,
+    data_loaders: Tuple[DataLoader, DataLoader],
 ) -> None:
     """
     run a comon learning pipeline
@@ -141,9 +150,11 @@ def learning_pipeline(
     :oaram data_loader: train and validation data loaders
     """
     trainer = create_supervised_trainer(
-        model, Adam(model.parameters(), lr=params["learning_rate"]), loss)
+        model, Adam(model.parameters(), lr=params["learning_rate"]), loss
+    )
     train_evaluator = create_supervised_evaluator(model, {"loss": Loss(loss)})
     evaluator = create_supervised_evaluator(model, {"loss": Loss(loss)})
+
     @trainer.on(Events.EPOCH_COMPLETED)
     # pylint: disable=unused-argument,unused-variable
     def validate(trainer):
@@ -152,34 +163,46 @@ def learning_pipeline(
 
     def score(engine):
         return -engine.state.metrics["loss"]
+
     early_stopping = EarlyStopping(10, score, trainer)
     evaluator.add_event_handler(Events.COMPLETED, early_stopping)
     checkpoint = ModelCheckpoint(
-        "checkpoints", "", score_function=score, require_empty=False)
+        "checkpoints", "", score_function=score, require_empty=False
+    )
     evaluator.add_event_handler(
-        Events.EPOCH_COMPLETED, checkpoint, {f"semigroup{cardinality}": model})
-    ProgressBar().attach(trainer, output_transform=lambda x: x,
-                         event_name=Events.EPOCH_COMPLETED,
-                         closing_event_name=Events.COMPLETED)
-    tb_logger = TensorboardLogger(log_dir=f"runs/{datetime.now()}")
+        Events.EPOCH_COMPLETED, checkpoint, {f"semigroup{cardinality}": model}
+    )
+    ProgressBar().attach(
+        trainer,
+        output_transform=lambda x: x,
+        event_name=Events.EPOCH_COMPLETED,
+        closing_event_name=Events.COMPLETED,
+    )
+    tb_logger = TensorboardLogger(
+        log_dir=f"runs/{datetime.now()}", flush_secs=1
+    )
     training_loss = OutputHandler(
-        "training", ["loss"],
-        global_step_transform=global_step_from_engine(trainer))
+        "training",
+        ["loss"],
+        global_step_transform=global_step_from_engine(trainer),
+    )
     tb_logger.attach(train_evaluator, training_loss, Events.COMPLETED)
     validation_loss = OutputHandler(
-        "validation", ["loss"],
-        global_step_transform=global_step_from_engine(trainer))
+        "validation",
+        ["loss"],
+        global_step_transform=global_step_from_engine(trainer),
+    )
     tb_logger.attach(evaluator, validation_loss, Events.COMPLETED)
     trainer.run(data_loaders[0], max_epochs=params["epochs"])
     tb_logger.close()
 
 
 def get_loaders(
-        cardinality: int,
-        batch_size: int,
-        train_size: int,
-        validation_size: int,
-        use_labels: bool = False
+    cardinality: int,
+    batch_size: int,
+    train_size: int,
+    validation_size: int,
+    use_labels: bool = False,
 ) -> Tuple[DataLoader, DataLoader]:
     """
     get train and validation data loaders
@@ -193,11 +216,13 @@ def get_loaders(
     :returns: a pair of train and validation data loaders
     """
     (
-        train, validation, _,
-        train_labels, validation_labels, _
-    ) = load_database_as_cubes(
-        cardinality, train_size, validation_size
-    )
+        train,
+        validation,
+        _,
+        train_labels,
+        validation_labels,
+        _,
+    ) = load_database_as_cubes(cardinality, train_size, validation_size)
     train_tensor = torch.from_numpy(train).to(CURRENT_DEVICE)
     val_tensor = torch.from_numpy(validation).to(CURRENT_DEVICE)
     if use_labels:
@@ -212,5 +237,5 @@ def get_loaders(
         val_data = TensorDataset(val_tensor, val_tensor)
     return (
         DataLoader(train_data, batch_size=batch_size, shuffle=True),
-        DataLoader(val_data, batch_size=batch_size, shuffle=True)
+        DataLoader(val_data, batch_size=batch_size, shuffle=True),
     )
